#include "testApp.h"


//--------------------------------------------------------------
void testApp::setup(){	
	
	ofBackground(252, 244, 233);
	
	ofSetVerticalSync(true);
	ofSetFrameRate(60);

	
	for (int i = 0; i < 20; i++)
	{
		vector <particle> particles;
		float shifterX = ofRandom(1,120);
		float shifterY = ofRandom(1,300);
		for (int j = 0; j < ofRandom(30, 60); j++)
		{
			particle myParticle;
		
			float x = 10 * shifterX * cos ( (j / 400.0) * TWO_PI);
			float y = 10 * shifterY * sin ( (j / 400.0) * TWO_PI);
		
			myParticle.setInitialCondition(x,y ,0,0);
			particles.push_back(myParticle);
		}
		blobs.push_back(particles);
	}
	
	
	for (int i = 0; i < blobs.size(); i++){
		vector <spring> springs;
		for (int j = 0; j < blobs[i].size(); j++)
		{
			spring mySpring;
			mySpring.distance		= 10;
			mySpring.springiness	= 0.2f;
			mySpring.particleA = & (blobs[i][j ]);
			mySpring.particleB = & (blobs[i][(j+1) % blobs[i].size()]);
			springs.push_back(mySpring);
		}
		blobsprings.push_back(springs);
	}
	
	
	for (int i = 0; i < blobs.size(); i++){
		vector <float> tex;
		for (int j = 0; j < blobs[i].size(); j++)
		{
			tex.push_back(img.getTextureReference().getCoordFromPoint(blobs[i][j].pos.x,blobs[i][j].pos.y).x);
			tex.push_back(img.getTextureReference().getCoordFromPoint(blobs[i][j].pos.x,blobs[i][j].pos.y).y);
		}
		texCoords.push_back(tex);
	}
	
	
		ofSoundStreamSetup(0,2,this, 44100, 256, 4);	
	left = new float[256];
	right = new float[256];
	
	bufferCounter = 0;
	drawCounter = 0;
	soundIn = 0;
	
		
	
}

//--------------------------------------------------------------
void testApp::update(){

	for (int i = 0; i < 256; i++)
	{
		soundIn = ofMap((right[i] + left[i])/2, 0, .3, 30,1000,true );
	}
	
	for (int i = 0; i < blobs.size(); i++){
		for (int j = 0; j < blobs[i].size(); j++)
		{
			blobs[i][j].resetForce();
		}
	}



	for (int i = 0; i < blobs.size(); i++){

		float it = ofMap(i, 0, blobs.size(),0, 256);
		float frc = ofMap((right[(int)it] + left[(int)it])/2, 0,.3, .1,5,true );
		float maxx,maxy = 0;
		float minx = ofGetWidth();
		float miny = ofGetHeight();
		for (int j = 0; j < blobs[i].size(); j++)
		{
			blobs[i][j].addRepulsionForce(mouseX, mouseY, 200, 0.7f);		

			float n = ofNoise(blobs[i][j].pos.x,blobs[i][j].pos.y);
	

			if ( frc >= 3) blobs[i][j].addRepulsionForce(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()), frc*3, frc);		
			if ( frc <= 1) 
			{	
				blobs[i][j].addAttractionForce(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()), frc*3, frc*3);		
			}
	
			for (int k = 0; k < j; k++)
			{
				blobs[i][j].addRepulsionForce(blobs[i][k], 50,n*.4);	
				blobs[i][j].addRepulsionForce(blobs[i][k], 40, frc);
	

			}
		}
	}

	for (int i = 0; i < blobsprings.size(); i++)
	{
		for (int j = 0; j < blobsprings[i].size(); j++)
		{
			blobsprings[i][j].update();
		}
	}


	for (int i = 0; i < blobs.size(); i++)
	{
		for (int j = 0; j < blobs[i].size(); j++)
		{
			blobs[i][j].bounceOffWalls();
			blobs[i][j].addDampingForce();
			blobs[i][j].update();
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){

	ofSetColor(0xffffff);
	ofEnableAlphaBlending();
	for (int i = 0; i < blobs.size(); i++)
	{	
		float it = ofMap(i, 0, blobs.size(),0, 256);
		float alpha = ofMap((right[(int)it] + left[(int)it])/2, 0,50,100,true );
		ofBeginShape();
		for (int j = 0; j < blobs[i].size(); j++)
		{	
			ofFill();
			ofSetColor(254, 187, 185,alpha);
			ofVertex(blobs[i][j].pos.x,blobs[i][j].pos.y);
//			blobs[i][j].draw();
		}
		ofEndShape(false);
	}
	
	
	for (int i = 0; i < blobsprings.size(); i++)
	{
		for (int j = 0; j < blobsprings[i].size(); j++)
		{
//			ofSetColor(30, 30, 30,80);

			ofNoFill();
			blobsprings[i][j].draw();
		}
	}
	ofDisableAlphaBlending();

}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){ 
	
	switch (key){
			
		case ' ':

			break;
	}
	
	
}

//--------------------------------------------------------------
void testApp::keyReleased  (int key){ 
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
	blobs[0][0].pos.set(mouseX, mouseY);

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	blobs[0][0].bFixed = true;
}

//--------------------------------------------------------------
void testApp::mouseReleased(){
	
	blobs[0][0].bFixed = false;
}

void testApp::audioReceived 	(float * input, int bufferSize, int nChannels)
{	
	// samples are "interleaved"
	for (int i = 0; i < bufferSize; i++){
		left[i] = input[i*2];
		right[i] = input[i*2+1];
	}
	bufferCounter++;
	
}

